@@ -1,16 +1,19 @@
-import typing
-import requests
 import logging
+import typing
+
+import requests
+
 from .settings import (
-    BASE_URL,
-    INDUSTRY_VALUES,
-    SECTOR_VALUES,
-    PERIOD_VALUES,
     EXCHANGE_VALUES,
-    TIME_DELTA_VALUES,
+    INDUSTRY_VALUES,
+    PERIOD_VALUES,
+    SECTOR_VALUES,
     SERIES_TYPE_VALUES,
     STATISTICS_TYPE_VALUES,
     TECHNICAL_INDICATORS_TIME_DELTA_VALUES,
+    TIME_DELTA_VALUES,
+    BASE_URL_v3,
+    BASE_URL_v4,
 )
 
 CONNECT_TIMEOUT = 5
@@ -21,15 +24,17 @@ logging.getLogger("requests").setLevel(logging.WARNING)
 logging.getLogger("urllib3").setLevel(logging.WARNING)
 
 
-def __return_json(path: str, query_vars: typing.Dict) -> typing.List:
+def __return_json_v3(
+    path: str, query_vars: typing.Dict
+) -> typing.Optional[typing.List]:
     """
-    Query URL for JSON response.
+    Query URL for JSON response for v3 of FMP API.
 
     :param path: Path after TLD of URL
     :param query_vars: Dictionary of query values (after "?" of URL)
     :return: JSON response
     """
-    url = f"{BASE_URL}{path}"
+    url = f"{BASE_URL_v3}{path}"
     return_var = None
     try:
         response = requests.get(
@@ -37,8 +42,58 @@ def __return_json(path: str, query_vars: typing.Dict) -> typing.List:
         )
         if len(response.content) > 0:
             return_var = response.json()
-        else:
+
+        if len(response.content) == 0 or (
+            isinstance(return_var, dict) and len(return_var.keys()) == 0
+        ):
             logging.warning("Response appears to have no data.  Returning empty List.")
+            return_var = []
+
+    except requests.Timeout:
+        logging.error(f"Connection to {url} timed out.")
+    except requests.ConnectionError:
+        logging.error(
+            f"Connection to {url} failed:  DNS failure, refused connection or some other connection related "
+            f"issue."
+        )
+    except requests.TooManyRedirects:
+        logging.error(
+            f"Request to {url} exceeds the maximum number of predefined redirections."
+        )
+    except Exception as e:
+        logging.error(
+            f"A requests exception has occurred that we have not yet detailed an 'except' clause for.  "
+            f"Error: {e}"
+        )
+
+    return return_var
+
+
+def __return_json_v4(
+    path: str, query_vars: typing.Dict
+) -> typing.Optional[typing.List]:
+    """
+    Query URL for JSON response for v4 of FMP API.
+
+    :param path: Path after TLD of URL
+    :param query_vars: Dictionary of query values (after "?" of URL)
+    :return: JSON response
+    """
+    url = f"{BASE_URL_v4}{path}"
+    return_var = None
+    try:
+        response = requests.get(
+            url, params=query_vars, timeout=(CONNECT_TIMEOUT, READ_TIMEOUT)
+        )
+        if len(response.content) > 0:
+            return_var = response.json()
+
+        if len(response.content) == 0 or (
+            isinstance(return_var, dict) and len(return_var.keys()) == 0
+        ):
+            logging.warning("Response appears to have no data.  Returning empty List.")
+            return_var = []
+
     except requests.Timeout:
         logging.error(f"Connection to {url} timed out.")
     except requests.ConnectionError:
